package java.util.concurrent;

import java.util.concurrent.locks.AbstractQueuedSynchronizer;
import java.util.ConcurrentModificationException;
import java.security.AccessController;
import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;
import java.security.AccessControlContext;
import java.util.concurrent.locks.Condition;
import java.util.HashSet;
import java.util.concurrent.locks.ReentrantLock;
import java.util.concurrent.atomic.AtomicInteger;

public class ThreadPoolExecutor extends AbstractExecutorService
{
    private final AtomicInteger ctl;
    private static final int COUNT_BITS = 29;
    private static final int CAPACITY = 536870911;
    private static final int RUNNING = -536870912;
    private static final int SHUTDOWN = 0;
    private static final int STOP = 536870912;
    private static final int TIDYING = 1073741824;
    private static final int TERMINATED = 1610612736;
    private final BlockingQueue<Runnable> workQueue;
    private final ReentrantLock mainLock;
    private final HashSet<ThreadPoolExecutor.Worker> workers;
    private final Condition termination;
    private int largestPoolSize;
    private long completedTaskCount;
    private volatile ThreadFactory threadFactory;
    private volatile RejectedExecutionHandler handler;
    private volatile long keepAliveTime;
    private volatile boolean allowCoreThreadTimeOut;
    private volatile int corePoolSize;
    private volatile int maximumPoolSize;
    private static final RejectedExecutionHandler defaultHandler;
    private static final RuntimePermission shutdownPerm;
    private final AccessControlContext acc;
    private static final boolean ONLY_ONE = true;
    
    private static int runStateOf(int n) {
        return n & 0xE0000000;
    }
    
    private static int workerCountOf(int n) {
        return n & 0x1FFFFFFF;
    }
    
    private static int ctlOf(int n, int n2) {
        return n | n2;
    }
    
    private static boolean runStateLessThan(int n, int n2) {
        return n < n2;
    }
    
    private static boolean runStateAtLeast(int n, int n2) {
        return n >= n2;
    }
    
    private static boolean isRunning(int n) {
        return n < 0;
    }
    
    private boolean compareAndIncrementWorkerCount(int expectedValue) {
        return this.ctl.compareAndSet(expectedValue, expectedValue + 1);
    }
    
    private boolean compareAndDecrementWorkerCount(int expectedValue) {
        return this.ctl.compareAndSet(expectedValue, expectedValue - 1);
    }
    
    private void decrementWorkerCount() {
        while (!this.compareAndDecrementWorkerCount(this.ctl.get())) {}
    }
    
    private void advanceRunState(int n) {
        while (true) {
            int value = this.ctl.get();
            if (!runStateAtLeast(value, n) && !this.ctl.compareAndSet(value, ctlOf(n, workerCountOf(value)))) {
                continue;
            }
            break;
        }
    }
    
    final void tryTerminate() {
        while (true) {
            int value = this.ctl.get();
            if (isRunning(value) || runStateAtLeast(value, 1073741824) || (runStateOf(value) == 0 && !this.workQueue.isEmpty())) {
                return;
            }
            if (workerCountOf(value) != 0) {
                this.interruptIdleWorkers(true);
                return;
            }
            ReentrantLock mainLock = this.mainLock;
            mainLock.lock();
            try {
                if (this.ctl.compareAndSet(value, ctlOf(1073741824, 0))) {
                    try {
                        this.terminated();
                    }
                    finally {
                        this.ctl.set(ctlOf(1610612736, 0));
                        this.termination.signalAll();
                    }
                    return;
                }
            }
            finally {
                mainLock.unlock();
            }
        }
    }
    
    private void checkShutdownAccess() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(ThreadPoolExecutor.shutdownPerm);
            ReentrantLock mainLock = this.mainLock;
            mainLock.lock();
            try {
                Iterator<ThreadPoolExecutor.Worker> iterator = this.workers.iterator();
                while (iterator.hasNext()) {
                    securityManager.checkAccess(((ThreadPoolExecutor.Worker)iterator.next()).thread);
                }
            }
            finally {
                mainLock.unlock();
            }
        }
    }
    
    private void interruptWorkers() {
        ReentrantLock mainLock = this.mainLock;
        mainLock.lock();
        try {
            Iterator<ThreadPoolExecutor.Worker> iterator = this.workers.iterator();
            while (iterator.hasNext()) {
                ((ThreadPoolExecutor.Worker)iterator.next()).interruptIfStarted();
            }
        }
        finally {
            mainLock.unlock();
        }
    }
    
    private void interruptIdleWorkers(boolean b) {
        ReentrantLock mainLock = this.mainLock;
        mainLock.lock();
        try {
            for (ThreadPoolExecutor.Worker worker : this.workers) {
                Thread thread = worker.thread;
                if (!thread.isInterrupted() && worker.tryLock()) {
                    try {
                        thread.interrupt();
                    }
                    catch (SecurityException ex) {}
                    finally {
                        worker.unlock();
                    }
                }
                if (b) {
                    break;
                }
                continue;
            }
        }
        finally {
            mainLock.unlock();
        }
    }
    
    private void interruptIdleWorkers() {
        this.interruptIdleWorkers(false);
    }
    
    final void reject(Runnable runnable) {
        this.handler.rejectedExecution(runnable, this);
    }
    
    void onShutdown() {
    }
    
    final boolean isRunningOrShutdown(boolean b) {
        int runState = runStateOf(this.ctl.get());
        return runState == -536870912 || (runState == 0 && b);
    }
    
    private List<Runnable> drainQueue() {
        BlockingQueue<Object> workQueue = this.workQueue;
        ArrayList<Runnable> list = new ArrayList();
        workQueue.drainTo(list);
        if (!workQueue.isEmpty()) {
            for (Runnable e : (Runnable[])workQueue.toArray(new Runnable[0])) {
                if (workQueue.remove(e)) {
                    list.add(e);
                }
            }
            return list;
        }
        return list;
    }
    
    private boolean addWorker(Runnable runnable, boolean b) {
        while (true) {
            int n = this.ctl.get();
            int runState = runStateOf(n);
            if (runState >= 0 && (runState != 0 || runnable != null || this.workQueue.isEmpty())) {
                return false;
            }
            while (true) {
                int workerCount = workerCountOf(n);
                if (workerCount >= 536870911 || workerCount >= (b ? this.corePoolSize : this.maximumPoolSize)) {
                    return false;
                }
                if (this.compareAndIncrementWorkerCount(n)) {
                    boolean n2 = false;
                    boolean n3 = false;
                    ThreadPoolExecutor.Worker e = null;
                    try {
                        e = new ThreadPoolExecutor.Worker(runnable);
                        Thread thread = e.thread;
                        if (thread != null) {
                            ReentrantLock mainLock = this.mainLock;
                            mainLock.lock();
                            try {
                                int runState2 = runStateOf(this.ctl.get());
                                if (runState2 < 0 || (runState2 == 0 && runnable == null)) {
                                    if (thread.isAlive()) {
                                        throw new IllegalThreadStateException();
                                    }
                                    this.workers.add(e);
                                    int size = this.workers.size();
                                    if (size > this.largestPoolSize) {
                                        this.largestPoolSize = size;
                                    }
                                    n3 = true;
                                }
                            }
                            finally {
                                mainLock.unlock();
                            }
                            if (!(n3 == 0)) {
                                thread.start();
                                n2 = true;
                            }
                        }
                    }
                    finally {
                        if (n2 == 0) {
                            this.addWorkerFailed(e);
                        }
                    }
                    return n2;
                }
                n = this.ctl.get();
                if (runStateOf(n) != runState) {
                    break;
                }
                continue;
            }
        }
    }
    
    private void addWorkerFailed(ThreadPoolExecutor.Worker o) {
        ReentrantLock mainLock = this.mainLock;
        mainLock.lock();
        try {
            if (o != null) {
                this.workers.remove(o);
            }
            this.decrementWorkerCount();
            this.tryTerminate();
        }
        finally {
            mainLock.unlock();
        }
    }
    
    private void processWorkerExit(ThreadPoolExecutor.Worker o, boolean b) {
        if (b) {
            this.decrementWorkerCount();
        }
        ReentrantLock mainLock = this.mainLock;
        mainLock.lock();
        try {
            this.completedTaskCount += o.completedTasks;
            this.workers.remove(o);
        }
        finally {
            mainLock.unlock();
        }
        this.tryTerminate();
        int value = this.ctl.get();
        if (runStateLessThan(value, 536870912)) {
            if (!b) {
                int n = this.allowCoreThreadTimeOut ? 0 : this.corePoolSize;
                if (n == 0 && !this.workQueue.isEmpty()) {
                    n = 1;
                }
                if (workerCountOf(value) >= n) {
                    return;
                }
            }
            this.addWorker(null, false);
        }
    }
    
    private Runnable getTask() {
        int n = 0;
        while (true) {
            int value = this.ctl.get();
            int runState = runStateOf(value);
            if (runState >= 0 && (runState >= 536870912 || this.workQueue.isEmpty())) {
                this.decrementWorkerCount();
                return null;
            }
            int workerCount = workerCountOf(value);
            int n2 = this.allowCoreThreadTimeOut || workerCount > this.corePoolSize;
            if ((workerCount > this.maximumPoolSize || (n2 && n != 0)) && (workerCount > 1 || this.workQueue.isEmpty())) {
                if (!this.compareAndDecrementWorkerCount(value)) {
                    continue;
                }
                return null;
            }
            else {
                try {
                    Runnable runnable = n2 ? ((Runnable)this.workQueue.poll(this.keepAliveTime, TimeUnit.NANOSECONDS)) : ((Runnable)this.workQueue.take());
                    if (runnable != null) {
                        return runnable;
                    }
                    n = 1;
                }
                catch (InterruptedException ex) {
                    n = 0;
                }
            }
        }
    }
    
    final void runWorker(ThreadPoolExecutor.Worker worker) {
        Thread currentThread = Thread.currentThread();
        Runnable runnable = worker.firstTask;
        worker.firstTask = null;
        worker.unlock();
        boolean n = true;
        try {
            while (runnable != null || (runnable = this.getTask()) != null) {
                worker.lock();
                if ((runStateAtLeast(this.ctl.get(), 536870912) || (Thread.interrupted() && runStateAtLeast(this.ctl.get(), 536870912))) && !currentThread.isInterrupted()) {
                    currentThread.interrupt();
                }
                try {
                    this.beforeExecute(currentThread, runnable);
                    Throwable t = null;
                    try {
                        runnable.run();
                    }
                    catch (RuntimeException ex) {
                        t = ex;
                        throw ex;
                    }
                    catch (Error error) {
                        t = error;
                        throw error;
                    }
                    catch (Throwable cause) {
                        t = cause;
                        throw new Error(cause);
                    }
                    finally {
                        this.afterExecute(runnable, t);
                    }
                }
                finally {
                    runnable = null;
                    ++worker.completedTasks;
                    worker.unlock();
                }
            }
            n = false;
        }
        finally {
            this.processWorkerExit(worker, (boolean)n);
        }
    }
    
    public ThreadPoolExecutor(int n, int n2, long n3, TimeUnit timeUnit, BlockingQueue<Runnable> blockingQueue) {
        this(n, n2, n3, timeUnit, blockingQueue, Executors.defaultThreadFactory(), ThreadPoolExecutor.defaultHandler);
    }
    
    public ThreadPoolExecutor(int n, int n2, long n3, TimeUnit timeUnit, BlockingQueue<Runnable> blockingQueue, ThreadFactory threadFactory) {
        this(n, n2, n3, timeUnit, blockingQueue, threadFactory, ThreadPoolExecutor.defaultHandler);
    }
    
    public ThreadPoolExecutor(int n, int n2, long n3, TimeUnit timeUnit, BlockingQueue<Runnable> blockingQueue, RejectedExecutionHandler rejectedExecutionHandler) {
        this(n, n2, n3, timeUnit, blockingQueue, Executors.defaultThreadFactory(), rejectedExecutionHandler);
    }
    
    public ThreadPoolExecutor(int corePoolSize, int maximumPoolSize, long n, TimeUnit timeUnit, BlockingQueue<Runnable> workQueue, ThreadFactory threadFactory, RejectedExecutionHandler handler) {
        this.ctl = new AtomicInteger(ctlOf(-536870912, 0));
        this.mainLock = new ReentrantLock();
        this.workers = new HashSet();
        this.termination = this.mainLock.newCondition();
        if (corePoolSize < 0 || maximumPoolSize <= 0 || maximumPoolSize < corePoolSize || n < 0L) {
            throw new IllegalArgumentException();
        }
        if (workQueue == null || threadFactory == null || handler == null) {
            throw new NullPointerException();
        }
        this.acc = ((System.getSecurityManager() == null) ? null : AccessController.getContext());
        this.corePoolSize = corePoolSize;
        this.maximumPoolSize = maximumPoolSize;
        this.workQueue = workQueue;
        this.keepAliveTime = timeUnit.toNanos(n);
        this.threadFactory = threadFactory;
        this.handler = handler;
        return;
    }
    
    @Override
    public void execute(Runnable runnable) {
        if (runnable == null) {
            throw new NullPointerException();
        }
        int n = this.ctl.get();
        if (workerCountOf(n) < this.corePoolSize) {
            if (this.addWorker(runnable, true)) {
                return;
            }
            n = this.ctl.get();
        }
        if (isRunning(n) && this.workQueue.offer(runnable)) {
            int value = this.ctl.get();
            if (!isRunning(value) && this.remove(runnable)) {
                this.reject(runnable);
            }
            else if (workerCountOf(value) == 0) {
                this.addWorker(null, false);
            }
        }
        else if (!this.addWorker(runnable, false)) {
            this.reject(runnable);
        }
        return;
    }
    
    @Override
    public void shutdown() {
        ReentrantLock mainLock = this.mainLock;
        mainLock.lock();
        try {
            this.checkShutdownAccess();
            this.advanceRunState(0);
            this.interruptIdleWorkers();
            this.onShutdown();
        }
        finally {
            mainLock.unlock();
        }
        this.tryTerminate();
    }
    
    @Override
    public List<Runnable> shutdownNow() {
        ReentrantLock mainLock = this.mainLock;
        mainLock.lock();
        List<Runnable> drainQueue;
        try {
            this.checkShutdownAccess();
            this.advanceRunState(536870912);
            this.interruptWorkers();
            drainQueue = this.drainQueue();
        }
        finally {
            mainLock.unlock();
        }
        this.tryTerminate();
        return drainQueue;
    }
    
    @Override
    public boolean isShutdown() {
        return !isRunning(this.ctl.get());
    }
    
    public boolean isTerminating() {
        int value = this.ctl.get();
        return !isRunning(value) && runStateLessThan(value, 1610612736);
    }
    
    @Override
    public boolean isTerminated() {
        return runStateAtLeast(this.ctl.get(), 1610612736);
    }
    
    @Override
    public boolean awaitTermination(long n, TimeUnit timeUnit) throws InterruptedException {
        long n2 = timeUnit.toNanos(n);
        ReentrantLock mainLock = this.mainLock;
        mainLock.lock();
        try {
            while (!runStateAtLeast(this.ctl.get(), 1610612736)) {
                if (n2 <= 0L) {
                    return false;
                }
                n2 = this.termination.awaitNanos(n2);
            }
            return true;
        }
        finally {
            mainLock.unlock();
        }
    }
    
    @Override
    protected void finalize() {
        if (System.getSecurityManager() == null || this.acc == null) {
            this.shutdown();
        }
        else {
            AccessController.doPrivileged(()/*  */ -> {
                this.shutdown();
                return null;
            }, this.acc);
        }
    }
    
    public void setThreadFactory(ThreadFactory threadFactory) {
        if (threadFactory == null) {
            throw new NullPointerException();
        }
        this.threadFactory = threadFactory;
        return;
    }
    
    public ThreadFactory getThreadFactory() {
        return this.threadFactory;
    }
    
    public void setRejectedExecutionHandler(RejectedExecutionHandler handler) {
        if (handler == null) {
            throw new NullPointerException();
        }
        this.handler = handler;
        return;
    }
    
    public RejectedExecutionHandler getRejectedExecutionHandler() {
        return this.handler;
    }
    
    public void setCorePoolSize(int corePoolSize) {
        if (corePoolSize < 0) {
            throw new IllegalArgumentException();
        }
        int a = corePoolSize - this.corePoolSize;
        if (workerCountOf(this.ctl.get()) > (this.corePoolSize = corePoolSize)) {
            this.interruptIdleWorkers();
        }
        else if (a > 0) {
            int min = Math.min(a, this.workQueue.size());
            while (min-- > 0 && this.addWorker(null, true) && !this.workQueue.isEmpty()) {}
        }
        return;
    }
    
    public int getCorePoolSize() {
        return this.corePoolSize;
    }
    
    public boolean prestartCoreThread() {
        return workerCountOf(this.ctl.get()) < this.corePoolSize && this.addWorker(null, true);
    }
    
    void ensurePrestart() {
        int workerCount = workerCountOf(this.ctl.get());
        if (workerCount < this.corePoolSize) {
            this.addWorker(null, true);
        }
        else if (workerCount == 0) {
            this.addWorker(null, false);
        }
    }
    
    public int prestartAllCoreThreads() {
        int n = 0;
        while (this.addWorker(null, true)) {
            ++n;
        }
        return n;
    }
    
    public boolean allowsCoreThreadTimeOut() {
        return this.allowCoreThreadTimeOut;
    }
    
    public void allowCoreThreadTimeOut(boolean allowCoreThreadTimeOut) {
        if (allowCoreThreadTimeOut && this.keepAliveTime <= 0L) {
            throw new IllegalArgumentException("Core threads must have nonzero keep alive times");
        }
        if (allowCoreThreadTimeOut != this.allowCoreThreadTimeOut && (this.allowCoreThreadTimeOut = allowCoreThreadTimeOut)) {
            this.interruptIdleWorkers();
        }
        return;
    }
    
    public void setMaximumPoolSize(int maximumPoolSize) {
        if (maximumPoolSize <= 0 || maximumPoolSize < this.corePoolSize) {
            throw new IllegalArgumentException();
        }
        if (workerCountOf(this.ctl.get()) > (this.maximumPoolSize = maximumPoolSize)) {
            this.interruptIdleWorkers();
        }
        return;
    }
    
    public int getMaximumPoolSize() {
        return this.maximumPoolSize;
    }
    
    public void setKeepAliveTime(long n, TimeUnit timeUnit) {
        if (n < 0L) {
            throw new IllegalArgumentException();
        }
        if (n == 0L && this.allowsCoreThreadTimeOut()) {
            throw new IllegalArgumentException("Core threads must have nonzero keep alive times");
        }
        long nanos = timeUnit.toNanos(n);
        long n2 = nanos - this.keepAliveTime;
        this.keepAliveTime = nanos;
        if (n2 < 0L) {
            this.interruptIdleWorkers();
        }
        return;
    }
    
    public long getKeepAliveTime(TimeUnit timeUnit) {
        return timeUnit.convert(this.keepAliveTime, TimeUnit.NANOSECONDS);
    }
    
    public BlockingQueue<Runnable> getQueue() {
        return this.workQueue;
    }
    
    public boolean remove(Runnable runnable) {
        int remove = this.workQueue.remove(runnable);
        this.tryTerminate();
        return remove;
    }
    
    public void purge() {
        BlockingQueue<Object> workQueue = this.workQueue;
        try {
            Iterator<Object> iterator = workQueue.iterator();
            while (iterator.hasNext()) {
                Runnable runnable = (Runnable)iterator.next();
                if (runnable instanceof Future && ((Future)runnable).isCancelled()) {
                    iterator.remove();
                }
                else {
                    continue;
                }
            }
        }
        catch (ConcurrentModificationException ex) {
            for (Object o : workQueue.toArray()) {
                if (o instanceof Future && ((Future)o).isCancelled()) {
                    workQueue.remove(o);
                }
            }
        }
        this.tryTerminate();
    }
    
    public int getPoolSize() {
        ReentrantLock mainLock = this.mainLock;
        mainLock.lock();
        try {
            return runStateAtLeast(this.ctl.get(), 1073741824) ? 0 : this.workers.size();
        }
        finally {
            mainLock.unlock();
        }
    }
    
    public int getActiveCount() {
        ReentrantLock mainLock = this.mainLock;
        mainLock.lock();
        try {
            int n = 0;
            Iterator<ThreadPoolExecutor.Worker> iterator = this.workers.iterator();
            while (iterator.hasNext()) {
                if (((ThreadPoolExecutor.Worker)iterator.next()).isLocked()) {
                    ++n;
                }
                else {
                    continue;
                }
            }
            return n;
        }
        finally {
            mainLock.unlock();
        }
    }
    
    public int getLargestPoolSize() {
        ReentrantLock mainLock = this.mainLock;
        mainLock.lock();
        try {
            return this.largestPoolSize;
        }
        finally {
            mainLock.unlock();
        }
    }
    
    public long getTaskCount() {
        ReentrantLock mainLock = this.mainLock;
        mainLock.lock();
        try {
            long completedTaskCount = this.completedTaskCount;
            for (ThreadPoolExecutor.Worker worker : this.workers) {
                completedTaskCount += worker.completedTasks;
                if (worker.isLocked()) {
                    ++completedTaskCount;
                }
                else {
                    continue;
                }
            }
            return completedTaskCount + (long)this.workQueue.size();
        }
        finally {
            mainLock.unlock();
        }
    }
    
    public long getCompletedTaskCount() {
        ReentrantLock mainLock = this.mainLock;
        mainLock.lock();
        try {
            long completedTaskCount = this.completedTaskCount;
            Iterator<ThreadPoolExecutor.Worker> iterator = this.workers.iterator();
            while (iterator.hasNext()) {
                completedTaskCount += ((ThreadPoolExecutor.Worker)iterator.next()).completedTasks;
            }
            return completedTaskCount;
        }
        finally {
            mainLock.unlock();
        }
    }
    
    @Override
    public String toString() {
        ReentrantLock mainLock = this.mainLock;
        mainLock.lock();
        long completedTaskCount;
        int i;
        int size;
        try {
            completedTaskCount = this.completedTaskCount;
            i = 0;
            size = this.workers.size();
            for (ThreadPoolExecutor.Worker worker : this.workers) {
                completedTaskCount += worker.completedTasks;
                if (worker.isLocked()) {
                    ++i;
                }
                else {
                    continue;
                }
            }
        }
        finally {
            mainLock.unlock();
        }
        int value = this.ctl.get();
        return super.toString() + "[" + (runStateLessThan(value, 0) ? "Running" : (runStateAtLeast(value, 1610612736) ? "Terminated" : "Shutting down")) + ", pool size = " + size + ", active threads = " + i + ", queued tasks = " + this.workQueue.size() + ", completed tasks = " + completedTaskCount + "]";
    }
    
    protected void beforeExecute(Thread thread, Runnable runnable) {
    }
    
    protected void afterExecute(Runnable runnable, Throwable t) {
    }
    
    protected void terminated() {
    }
    
    static {
        defaultHandler = new ThreadPoolExecutor.AbortPolicy();
        shutdownPerm = new RuntimePermission("modifyThread");
    }
    
    private final class Worker extends AbstractQueuedSynchronizer implements Runnable
    {
        private static final long serialVersionUID = 6138294804551838833L;
        final Thread thread;
        Runnable firstTask;
        volatile long completedTasks;
        
        Worker(Runnable firstTask) {
            this.setState(-1);
            this.firstTask = firstTask;
            this.thread = ThreadPoolExecutor.this.getThreadFactory().newThread(this);
        }
        
        @Override
        public void run() {
            ThreadPoolExecutor.this.runWorker(this);
        }
        
        @Override
        protected boolean isHeldExclusively() {
            return this.getState() != 0;
        }
        
        @Override
        protected boolean tryAcquire(int n) {
            if (this.compareAndSetState(0, 1)) {
                this.setExclusiveOwnerThread(Thread.currentThread());
                return true;
            }
            return false;
        }
        
        @Override
        protected boolean tryRelease(int n) {
            this.setExclusiveOwnerThread(null);
            this.setState(0);
            return true;
        }
        
        public void lock() {
            this.acquire(1);
        }
        
        public boolean tryLock() {
            return this.tryAcquire(1);
        }
        
        public void unlock() {
            this.release(1);
        }
        
        public boolean isLocked() {
            return this.isHeldExclusively();
        }
        
        void interruptIfStarted() {
            Thread thread;
            if (this.getState() >= 0 && (thread = this.thread) != null && !thread.isInterrupted()) {
                try {
                    thread.interrupt();
                }
                catch (SecurityException ex) {}
            }
        }
    }
    
    public static class CallerRunsPolicy implements RejectedExecutionHandler
    {
        @Override
        public void rejectedExecution(Runnable runnable, ThreadPoolExecutor threadPoolExecutor) {
            if (!threadPoolExecutor.isShutdown()) {
                runnable.run();
            }
        }
    }
    
    public static class AbortPolicy implements RejectedExecutionHandler
    {
        @Override
        public void rejectedExecution(Runnable runnable, ThreadPoolExecutor threadPoolExecutor) {
            throw new RejectedExecutionException("Task " + runnable.toString() + " rejected from " + threadPoolExecutor.toString());
        }
    }
    
    public static class DiscardPolicy implements RejectedExecutionHandler
    {
        @Override
        public void rejectedExecution(Runnable runnable, ThreadPoolExecutor threadPoolExecutor) {
        }
    }
    
    public static class DiscardOldestPolicy implements RejectedExecutionHandler
    {
        @Override
        public void rejectedExecution(Runnable runnable, ThreadPoolExecutor threadPoolExecutor) {
            if (!threadPoolExecutor.isShutdown()) {
                threadPoolExecutor.getQueue().poll();
                threadPoolExecutor.execute(runnable);
            }
        }
    }
}
